<?php

namespace Drupal\view_custom_table\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Session\AccountProxy;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Add views custom table form.
 */
class AddViewsCustomTable extends FormBase {
  /**
   * Step of the form.
   *
   * @var int
   */
  protected $step;

  /**
   * Previous step form data.
   *
   * @var array
   */
  protected $PreviousStepData;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $account;

  /**
   * Class constructor.
   */
  public function __construct(AccountProxy $account) {
    $this->step = 1;
    $this->PreviousStepData = [];
    $this->account = $account;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'view_custom_table_add_table_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    if ($this->step == 1) {
      $form['table_name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Table'),
        '#required' => TRUE,
      ];
      $form['description'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Description'),
        '#rows' => 5,
        '#description' => $this->t('Maximum 255 letters are allowed.'),
      ];

      $form['actions'] = [
        '#type' => 'actions',
      ];
      $form['actions']['next'] = [
        '#type' => 'submit',
        '#value' => $this->t('Next'),
      ];
    }
    elseif ($this->step == 2) {
      $table_name = $this->PreviousStepData['table_name'];
      $form['columns'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('"@table" Int Type Columns', [
          '@table' => $table_name,
        ]),
        '#tree' => TRUE,
      ];

      $all_entities = \Drupal::entityTypeManager()->getDefinitions();
      $entities['none'] = $this->t('None');
      foreach ($all_entities as $entity_name => $entity) {
        if ($entity->getBaseTable()) {
          $entities[$entity_name] = $entity->getLabel()->render();
        }
      }
      $all_custom_tables = view_custom_table_load_tables_info($table_name);
      if (!empty($all_custom_tables)) {
        foreach ($all_custom_tables as $custom_table) {
          $entities[$custom_table->table_name] = $custom_table->table_name;
        }
      }
      $int_types = ['tinyint', 'smallint', 'mediumint', 'int', 'bigint'];
      $text_query = format_string('DESCRIBE { @table_name }', [
        '@table_name' => $table_name,
      ]);
      $query = db_query($text_query);
      foreach ($query as $row) {
        $row_type = explode('(', $row->Type);
        if (in_array($row_type[0], $int_types)) {
          $form['columns']['column_' . $row->Field] = [
            '#type' => 'fieldset',
            '#title' => $this->t('Relation of "@field_name" with', [
              '@field_name' => ucfirst($row->Field),
            ]),
            '#tree' => TRUE,
            '#attributes' => [
              'class' => [
                'container-inline',
              ],
            ],
          ];
          $form['columns']['column_' . $row->Field]['entity'] = [
            '#type' => 'select',
            '#title' => $this->t('Entity'),
            '#options' => $entities,
          ];
          $form['columns']['column_' . $row->Field]['field'] = [
            '#type' => 'hidden',
            '#value' => $row->Field,
          ];
        }
      }
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Save'),
      ];
    }
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->buildCancelLinkUrl(),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if ($this->step == 1) {
      $table_name = $form_state->getValue('table_name');
      $description = $form_state->getValue('description');
      if (!\Drupal::database()->schema()->tableExists($table_name)) {
        $form_state->setErrorByName('table_name', $this->t('@table not found in database, please check table name, and database again.', [
          '@table' => $table_name,
        ]));
      }
      if (\Drupal::database()->schema()->tableExists('custom_table_view_data')) {
        $query = \Drupal::database()->select('custom_table_view_data', 'vd')
          ->fields('vd', [
            'id',
          ])
          ->condition('table_name', $table_name);
        $table_exist = $query->execute()->fetchField();
        if ($table_exist) {
          $form_state->setErrorByName('table_name', $this->t("@table is already available for views. If you can't find it, please clear cache and try again.", [
            '@table' => $table_name,
          ]));
        }
      }
      if (strlen($description) > 254) {
        $form_state->setErrorByName('description', $this->t("Description can not be more then 255 letters. Please update it and try again.", [
          '@table' => $table_name,
        ]));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $opeation = $form_state->getValue('op')->render();
    if ($opeation == $this->t('Next')) {
      $this->PreviousStepData = $form_state->cleanValues()->getValues();
      $form_state->setRebuild();
      $this->step++;
    }
    if ($opeation == $this->t('Save')) {
      $user = $this->account;
      $table_name = $this->PreviousStepData['table_name'];
      $description = $this->PreviousStepData['description'];
      $relations = $form_state->getValue('columns');
      $column_relations = [];
      foreach ($relations as $relation) {
        if ($relation['entity'] != 'none') {
          $column_relations[$relation['field']] = $relation['entity'];
        }
      }
      $serialize_relations = serialize($column_relations);

      $result = \Drupal::database()->insert('custom_table_view_data')
        ->fields([
          'table_name' => $table_name,
          'description' => $description,
          'column_relations' => $serialize_relations,
          'created_by' => $user->id(),
        ])
        ->execute();
      if ($result) {
        drupal_set_message($this->t('@table is added to views. Please clear cache to see changes.', [
          '@table' => $table_name,
        ]));
      }
      else {
        drupal_set_message($this->t('Could not add table to views, please check log messages for error.'), 'error');
      }
    }
    $form_state->setRedirect('view_custom_table.customtable');
  }

  /**
   * Builds the cancel link url for the form.
   *
   * @return Drupal\Core\Url
   *   Cancel url
   */
  private function buildCancelLinkUrl() {
    $query = $this->getRequest()->query;

    if ($query->has('destination')) {
      $options = UrlHelper::parse($query->get('destination'));
      $url = Url::fromUri('internal:/' . $options['path'], $options);
    }
    else {
      $url = Url::fromRoute('view_custom_table.customtable');
    }

    return $url;
  }

}
